#include "btree.h"

/*
 * The function fetch_page reads a page from disk using the formula
 * page_num * PAGE_SIZE - PAGE_SIZE, e.g., if the page_num = 3 and
 * PAGE_SIZE = 1024, the B-Tree file will be accessed starting at
 * offset 2048, wherein the first page would be stored at offset 0,
 * and the second page at offset 1024.
 *
 * fetch_page installs the page in a dynamically allocated PageHdr
 * structure, which has a pointer to a linked-list of keys stored in 
 * dynamically allocated KeyRecord structures.
 */
PageHdr* fetch_page(long page_num) {

    /* check validity of "page_num" */
    long num_pages_in_tree = ffsize(fp_btree) / PAGE_SIZE;
    if ((page_num < 1) || (page_num > num_pages_in_tree)) {
        printf("fetch_page: Pagenum %d out of range (%d,%d)\n",
               (int) page_num, (int) ROOT, (int) num_pages_in_tree);
        /* exit(-1); */
    }

    /* Read in the page_num header */
    PageHdr* page_ptr = (PageHdr*) malloc(sizeof(*page_ptr));
    check_malloc(page_ptr, "page_ptr");
    fseek(fp_btree, page_num * PAGE_SIZE - PAGE_SIZE, 0);

    fread(&page_ptr->page_type, sizeof(char), 1, fp_btree);
    fread(&page_ptr->page_num, sizeof(page_ptr->page_num), 1, fp_btree);
    if ((page_ptr->page_num) != page_num) {
        printf("fetch_page: corrupted page_num %d\n", (int) page_num);
        exit(-1);
    }

    if (is_leaf(page_ptr))
        fread(&page_ptr->next_leaf_page_num, sizeof(page_ptr->next_leaf_page_num), 1, fp_btree);
    fread(&page_ptr->num_bytes, sizeof(page_ptr->num_bytes), 1, fp_btree);
    fread(&page_ptr->num_keys, sizeof(page_ptr->num_keys), 1, fp_btree);
    page_ptr->key_list_ptr = NULL;
    if (!is_leaf(page_ptr))
        fread(&page_ptr->final_right_page_num, sizeof(page_ptr->final_right_page_num), 1, fp_btree);

    /* Read in the keys */
    KeyRecord* key_list_iterator = NULL;  /* To traverse the list of keys */
    KeyRecord* key_record;
    for (int i = 0; i < page_ptr->num_keys; i++) {
        key_record = (KeyRecord*) malloc(sizeof(*key_record));
        check_malloc(key_record, "key_record");
        if (!is_leaf(page_ptr))
            fread(&key_record->page_num, sizeof(key_record->page_num), 1, fp_btree);
        fread(&key_record->key_len, sizeof(key_record->key_len), 1, fp_btree);
        key_record->stored_key = (char*) malloc((key_record->key_len) + 1);
        check_malloc(key_record->stored_key, "key_record->stored_key in fetch_page()");
        fread(key_record->stored_key, sizeof(char), key_record->key_len, fp_btree);
        (*(key_record->stored_key + key_record->key_len)) = '\0';  /* string terminator */
        if (is_leaf(page_ptr))
            fread(&key_record->posting, sizeof(key_record->posting), 1, fp_btree);
        if (key_list_iterator == NULL) {
            key_list_iterator = key_record;
            page_ptr->key_list_ptr = key_record;
        } else {
            key_list_iterator->next = key_record;
            key_list_iterator = key_list_iterator->next;
        }
    }
    if (page_ptr->num_keys != 0)
        key_list_iterator->next = NULL;

    return page_ptr;
}
