#include "btree.h"

/*
 * This function writes a page to disk and then calls function
 * free_page to free the storage space that was allocated to the
 * dynamic data structures.
 *
 * The size of the page, PAGE_SIZE, is locked-in when the page is
 * written to disk. This is handled by the padding of character
 * zero's, e.g., if PAGE_SIZE = x and the number of bytes that are
 * stored in the page is num_bytes = n, there will be (x - n)
 * character byte zero's written to disk.
 */
void flush_page(PageHdr* page_ptr) {
    update_page_header(page_ptr);  /* calculates the # of bytes and # of keys */
    if (page_ptr->num_bytes > PAGE_SIZE) {
        printf("flush_page: page %d too big: %d bytes\n",
               (int) page_ptr->page_num, (int) page_ptr->num_bytes);
        printf("execution aborted\n");
        exit(-2);
    }

    fseek(fp_btree, (page_ptr->page_num) * PAGE_SIZE - PAGE_SIZE, 0);
#ifdef DEBUG

    printf("flush_page starting: ftell()=%ld for page # %d\n", ftell(fp_btree),
    (int) (page_ptr->page_num));
#endif

    /* Write the page header to disk */
    fwrite(&page_ptr->page_type, sizeof(page_ptr->page_type), 1, fp_btree);
    fwrite(&page_ptr->page_num, sizeof(page_ptr->page_num), 1, fp_btree);
    if (is_leaf(page_ptr))
        fwrite(&page_ptr->next_leaf_page_num, sizeof(page_ptr->next_leaf_page_num), 1, fp_btree);
    fwrite(&page_ptr->num_bytes, sizeof(page_ptr->num_bytes), 1, fp_btree);
    fwrite(&page_ptr->num_keys, sizeof(page_ptr->num_keys), 1, fp_btree);
    if (!is_leaf(page_ptr))
        fwrite(&page_ptr->final_right_page_num, sizeof(page_ptr->final_right_page_num), 1, fp_btree);

    /* Write the keys to disk */
    KeyRecord* key_list_iterator;  /* A pointer to the list of keys */
    for (key_list_iterator = page_ptr->key_list_ptr;
         key_list_iterator != NULL;
         key_list_iterator = key_list_iterator->next) {
        if (!is_leaf(page_ptr))
            fwrite(&key_list_iterator->page_num, sizeof(key_list_iterator->page_num), 1, fp_btree);
        fwrite(&key_list_iterator->key_len, sizeof(key_list_iterator->key_len), 1, fp_btree);
        fwrite(key_list_iterator->stored_key, sizeof(char), key_list_iterator->key_len, fp_btree);
        if (is_leaf(page_ptr))
            fwrite(&key_list_iterator->posting, sizeof(key_list_iterator->posting), 1, fp_btree);
    }

    /* Lock-in full page size */
    char c = '0';
    for (int i = page_ptr->num_bytes; i < PAGE_SIZE; i++)
        fwrite(&c, sizeof(c), 1, fp_btree);

#ifdef DEBUG

    printf("flush_page ending: ftell()=%ld for page # %d\n", ftell(fp_btree),
    (int) (page_ptr->page_num));
#endif

    fflush(fp_btree);  /* make sure the page is written back */

    /* Free core storage allocated for page header and keys */
    free_page(page_ptr);
}
